unit MainUnit;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, LResources, Forms, Controls, Graphics, Dialogs, Dbf, DB,
  Menus, ActnList, ExtCtrls, DBCtrls, DBGrids, StdCtrls,
  Printers, Buttons, AboutUnit,LR_Class, LR_DBSet;

type
  TDataSetFirst = TAction;
  TDataSetPrior = TAction;
  TDataSetNext = TAction;
  TDataSetLast = TAction;
  TDataSetInsert = TAction;
  TDataSetDelete = TAction;
  TDataSetEdit = TAction;
  TDataSetPost = TAction;
  TDataSetCancel = TAction;
  TDataSetRefresh = TAction;
  
  { TContactForm }

  TContactForm = class(TForm)
    AClose: TAction;
    ACancel: TDataSetCancel;
    APrint: TAction;
    ARefresh: TDataSetRefresh;
    APost: TDataSetPost;
    AEdit: TDataSetEdit;
    ADelete: TDataSetDelete;
    AInsert: TDataSetInsert;
    ALast: TDataSetLast;
    ANext: TDataSetNext;
    APrior: TDataSetPrior;
    AFirst: TDataSetFirst;
    AQuit: TAction;
    AOpen: TAction;
    ANew: TAction;
    ActionList1: TActionList;
    Button1: TButton;
    Button2: TButton;
    CancelButton: TButton;
    frDBDataSet1: TfrDBDataSet;
    ContactReport: TfrReport;
    PostButton: TButton;
    EditButton: TButton;
    InsertButton: TButton;
    LastButton: TButton;
    NextButton: TButton;
    PriorButton: TButton;
    FirstButton: TButton;
    ContactSource: TDatasource;
    Address1DBEdit: TDBEdit;
    Address2DBEdit: TDBEdit;
    Label15: TLabel;
    CurrentRecordLabel: TLabel;
    NumOfRecordsLabel: TLabel;
    LastNameDBEdit: TDBEdit;
    TelephoneDBEdit: TDBEdit;
    FaxDBEdit: TDBEdit;
    MobileDBEdit: TDBEdit;
    EmailDBEdit: TDBEdit;
    TelephoneLabel: TLabel;
    FaxLabel: TLabel;
    MobileLabel: TLabel;
    WWWDBEdit: TDBEdit;
    EmailLabel: TLabel;
    WWWLabel: TLabel;
    TownDBEdit: TDBEdit;
    CountyDBEdit: TDBEdit;
    PostcodeDBEdit: TDBEdit;
    TitleDBEdit: TDBEdit;
    FirstDBEdit: TDBEdit;
    FirstNameLabel: TLabel;
    TitleLabel: TLabel;
    Address1Label: TLabel;
    Address2Label: TLabel;
    TownLabel: TLabel;
    CountyLabel: TLabel;
    PostcodeLabel: TLabel;
    MydbGrid: TdbGrid;
    CommentDBMemo: TDBMemo;
    DBNavigator1: TDBNavigator;
    LastNameLabel: TLabel;
    CommentsLabel: TLabel;
    MainMenu1: TMainMenu;
    FileMenu: TMenuItem;
    MenuItem1: TMenuItem;
    MenuItem2: TMenuItem;
    ExitMenuItem: TMenuItem;
    FirstRecordMenuItem: TMenuItem;
    LastRecordMenuItem: TMenuItem;
    InsertRecordMenuItem: TMenuItem;
    DeleteRecordMenuItem: TMenuItem;
    EditRecordMenuItem: TMenuItem;
    CancelRecordMenuItem: TMenuItem;
    AboutMenuItem: TMenuItem;
    Panel1: TPanel;
    Panel2: TPanel;
    RefreshRecordMenuItem: TMenuItem;
    PostRecordMenuItem: TMenuItem;
    NextRecordMenuItem: TMenuItem;
    PriorRecordMenuItem: TMenuItem;
    RecordMenu: TMenuItem;
    HelpMenu: TMenuItem;
    PrintMenuItem: TMenuItem;
    SaveMenuItem: TMenuItem;
    OpenMenuItem: TMenuItem;
    NewMenuItem: TMenuItem;
    MyDbf: TDbf;
    OpenDataDialog: TOpenDialog;
    SaveDataDialog: TSaveDialog;
    procedure AboutMenuItemClick(Sender: TObject);
    procedure ANewExecute(Sender : TObject);
    procedure ACloseExecute(Sender : TObject);
    procedure AQuitExecute(Sender : TObject);
    procedure ACloseUpdate(Sender : TObject);
    procedure AOpenExecute(Sender : TObject);
    procedure DataOpen(Sender : TObject);
    procedure DoDataAction(Sender : TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: boolean);
    procedure FormCreate(Sender: TObject);
    procedure HaveDataNotEmpty(Sender : TObject);
    procedure InEditMode(Sender : TObject);
    procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure PrintMenuItemClick(Sender: TObject);
  private
    { private declarations }
    procedure CreateTable(AFileName : String);
    procedure OpenTable(AFileName : string);
    procedure CloseTable;
    procedure PrintWholeFile;
    procedure PrintCurrentRecord;
  public
    { public declarations }
  end; 

var
  ContactForm: TContactForm;

implementation

resourcestring
     SSaveData = 'There is still unsaved data. Save now ?';
     PPrintMess ='Press "Yes" to print out the whole list, or "No" for the current record';
     
{ANewExecute creates a new table (FileName) and enables the Print menu item}
procedure TContactForm.ANewExecute(Sender : TObject);
begin
    with SaveDataDialog do
         if Execute then
         begin
            CreateTable(FileName);
            PrintMenuItem.Enabled := true;
         end;
end;

{Shows the AboutBox}
procedure TContactForm.AboutMenuItemClick(Sender: TObject);
begin
  AboutForm.ShowModal;
end;

{Closes an open database and disables the Print Menu item}
procedure TContactForm.ACloseExecute(Sender : TObject);
begin
     CloseTable;
     CommentDBMemo.Clear;
     NumOfRecordsLabel.Caption := 'Number of Records : 0';
     CurrentRecordLabel.Caption := 'Current Record : 0';
     PrintMenuItem.Enabled := false;
end;

procedure TContactForm.ACloseUpdate(Sender : TObject);
begin
     (Sender as TAction).Enabled := Not (MyDbf.State in dsEditModes);
end;

{Takes the file (FileName) and passes it to the OpenTable procedure
 to open the Contacts database}
procedure TContactForm.AOpenExecute(Sender : TObject);
begin
    with OpenDataDialog do
         if Execute then
            OpenTable(FileName);
end;

{Opens an existing Contacts database and enables the Print menu item}
procedure TContactForm.OpenTable(AFileName : string);
begin
    with MyDbf do
    begin
         Close;
         TableName := AFileName;
         Open;
    end;
    NumOfRecordsLabel.Caption := 'Number of Records : ' + IntToStr(MyDbf.RecordCount);
    CurrentRecordLabel.Caption := 'Current Record : ' + IntToStr(MyDbf.PhysicalRecNo);
    PrintMenuItem.Enabled := true;
end;


{Creates a new Contacts database with the name in (AFileName)}
procedure TContactForm.CreateTable(AFileName : String);
begin
     with MyDbf do
     begin
          Close;
          with FieldDefs do
          begin
               Clear;
               Add('Title', ftString,10);
               Add('FirstName', ftString,30);
               Add('LastName', ftString,30);
               Add('Address1', ftString,50);
               Add('Address2', ftString,50);
               Add('Town', ftString,30);
               Add('County', ftString,30);
               Add('PostCode', ftString,20);
               Add('Telephone', ftString,15);
               Add('Fax', ftString,15);
               Add('Mobile', ftString,15);
               Add('Email', ftString,40);
               Add('WWW', ftString,40);
               Add('Comments', ftMemo,0);
          end;
          TableName := AFileName;
          CreateTable;
          Exclusive := True;
          Open;
          AddIndex('LastName', 'LastName', []);
     end;
end;

{Closes an open table}
procedure TContactForm.CloseTable;
begin
     MyDbf.Close;
end;

{Exits the application}
procedure TContactForm.AQuitExecute(Sender : TObject);
begin
    Close;
end;

procedure TContactForm.DataOpen(Sender : TObject);
begin
    with MyDbf do
         (Sender as TAction).Enabled := Active;

    NumOfRecordsLabel.Caption := 'Number of Records : ' + IntToStr(MyDbf.RecordCount);
     CurrentRecordLabel.Caption := 'Current Record : ' + IntToStr(MyDbf.PhysicalRecNo);
end;

procedure TContactForm.FormClose(Sender: TObject; var CloseAction: TCloseAction);
begin
  CloseTable;
end;

{Asks the user if they want to print the whole database or just the current record}
{ If "Yes" is pressed, a preview report sheet is shown and the user can print it from there.}
{ If "No" is pressed the current record is printed on a single sheet of paper.}
procedure TContactForm.PrintMenuItemClick(Sender: TObject);
begin
  case MessageDlg(PPrintMess,mtInformation, [mbYes,mbNo, mbCancel],0) of
       mrYes : PrintWholeFile;
       mrNo  : PrintCurrentRecord;
  end;
end;

procedure TContactForm.DoDataAction(Sender : TObject);
begin
     case (Sender as TAction).Tag of
          0 : MyDbf.First;
          1 : MyDbf.Prior;
          2 : MyDbf.Next;
          3 : MyDbf.Last;
          4 : MyDbf.Insert;
          5 : MyDbf.Delete;
          6 : MyDbf.Edit;
          7 : MyDbf.Post;
          8 : MyDbf.Cancel;
          9 : MyDbf.Refresh;
     end;
     NumOfRecordsLabel.Caption := 'Number of Records : ' + IntToStr(MyDbf.RecordCount);
     CurrentRecordLabel.Caption := 'Current Record : ' + IntToStr(MyDbf.PhysicalRecNo);
end;

procedure TContactForm.FormCloseQuery(Sender: TObject; var CanClose: boolean);
begin
  CanClose := Not (MyDbf.State in dsEditModes);
  if Not CanClose then
     case MessageDlg(SSaveData, mtWarning, [mbYes, mbNo, mbCancel], 0) of
          mrYes : begin
                       MyDbf.Post;
                       CanClose := True;
                  end;
          mrNo  : CanClose := True;
     end;
end;

procedure TContactForm.FormCreate(Sender: TObject);
begin
    CommentDBMemo.Clear;

end;

procedure TContactForm.HaveDataNotEmpty(Sender : TObject);
begin
    with MyDbf do
         (Sender as TAction).Enabled := Active and Not (EOF and BOF) ;
end;

procedure TContactForm.InEditMode(Sender : TObject);
begin
    with MyDbf do
         (Sender as TAction).Enabled := State in dsEditModes;

end;

{Previews the entire database  and allows the user to print from there.
 This uses the LazReport components.}
procedure TContactForm.PrintWholeFile;
begin
     ContactReport.LoadFromFile(ExtractFilePath(ParamStr(0)) + 'contact.lrf');
     ContactReport.ShowReport;
end;

{Prints the current record on 1 sheet of paper}
procedure TContactForm.PrintCurrentRecord;
var
   r : TRect;
   i : integer;
begin
     with Printer do
     begin
          BeginDoc;
          Canvas.Brush.Style := bsClear;
          Canvas.Font.Size := 12;

          Canvas.Font.Italic := true;
          Canvas.TextOut(10,10,TitleLabel.Caption);
          Canvas.Font.Italic := false;
          Canvas.TextOut(100,10,TitleDBEdit.Text);
          
          Canvas.Font.Italic := true;
          Canvas.TextOut(10,30,LastNameLabel.Caption);
          Canvas.Font.Italic := false;
          Canvas.TextOut(100,30,LastNameDBEdit.Text);
          
          Canvas.Font.Italic := true;
          Canvas.TextOut(10,50,FirstNameLabel.Caption);
          Canvas.Font.Italic := false;
          Canvas.TextOut(100,50,FirstDBEdit.Text);
          
          Canvas.Font.Italic := true;
          Canvas.TextOut(10,70,Address1Label.Caption);
          Canvas.Font.Italic := false;
          Canvas.TextOut(100,70,Address1DBEdit.Text);
          
          Canvas.Font.Italic := true;
          Canvas.TextOut(10,90,Address2Label.Caption);
          Canvas.Font.Italic := false;
          Canvas.TextOut(100,90,Address2DBEdit.Text);
          
          Canvas.Font.Italic := true;
          Canvas.TextOut(10,110,TownLabel.Caption);
          Canvas.Font.Italic := false;
          Canvas.TextOut(100,110,TownDBEdit.Text);
          
          Canvas.Font.Italic := true;
          Canvas.TextOut(10,130,CountyLabel.Caption);
          Canvas.Font.Italic := false;
          Canvas.TextOut(100,130,CountyDBEdit.Text);
          
          Canvas.Font.Italic := true;
          Canvas.TextOut(10,150,PostcodeLabel.Caption);
          Canvas.Font.Italic := false;
          Canvas.TextOut(100,150,PostcodeDBEdit.Text);
          
          Canvas.Font.Italic := true;
          Canvas.TextOut(10,170,EmailLabel.Caption);
          Canvas.Font.Italic := false;
          Canvas.TextOut(100,170,EmailDBEdit.Text);
          
          Canvas.Font.Italic := true;
          Canvas.TextOut(10,190,WWWLabel.Caption);
          Canvas.Font.Italic := false;
          Canvas.TextOut(100,190,WWWDBEdit.Text);
          
          Canvas.Font.Italic := true;
          Canvas.TextOut(10,210,TelephoneLabel.Caption);
          Canvas.Font.Italic := false;
          Canvas.TextOut(100,210,TelephoneDBEdit.Text);
          
          Canvas.Font.Italic := true;
          Canvas.TextOut(10,230,FaxLabel.Caption);
          Canvas.Font.Italic := false;
          Canvas.TextOut(100,2300,FaxDBEdit.Text);
          
          Canvas.Font.Italic := true;
          Canvas.TextOut(10,250,MobileLabel.Caption);
          Canvas.Font.Italic := false;
          Canvas.TextOut(100,250,MobileDBEdit.Text);
          
          Canvas.Font.Italic := true;
          Canvas.TextOut(10,270,CommentsLabel.Caption);
          Canvas.Font.Italic := false;
          
          r := Rect(100,270,(PageWidth - 200), (PageHeight - 200));
          Canvas.Font.Size := 10;
          for i := 0 to CommentDBMemo.Lines.Count do
              Canvas.TextOut(100,270 + (i * Canvas.TextHeight(CommentDBMemo.Lines.Strings[i])), CommentDBMemo.Lines.Strings[i]);
          Canvas.Brush.Color := clBlack;
          Canvas.FrameRect(r);
          EndDoc;
     end;
end;

initialization
  {$I mainunit.lrs}

end.

